use gtk4 :: Image ;
use gtk4 :: ListBoxRow ;
use gtk4 :: Align :: * ;
use libadwaita :: * ;
use std :: cell :: * ;

use crate :: utility :: * ;
use crate :: ui :: * ;
use crate :: ui :: utility :: * ;
use crate :: ui :: collatable_container :: SortingDirection :: * ;



pub struct FilmSortButton {
	widget : SplitButton ,
	previous_sorting : & 'static RefCell <FilmsSorting> ,
}
pub struct SeriesSortButton {
	widget : SplitButton ,
	previous_sorting : & 'static RefCell <SeriesSorting> ,
}

impl FilmSortButton {
	pub fn new < F : Fn (FilmsSorting) + 'static > ( on_sort : F ) -> Self {
		let previous_sorting = leak ( RefCell :: new ( FilmsSorting :: default () ) ) ;

		let sort_icons = leak ( [
			image ! ( @ icon_name : "view-sort-ascending-symbolic" ; ) ,
			image ! ( @ icon_name : "view-sort-ascending-symbolic" ; ) ,
			image ! ( @ icon_name : "view-sort-ascending-symbolic" ; ) ,
		] ) ;

		let widget = split_button ! (
			@ popover : & popover ! (
				@ css_classes : & [ "menu" ] ;
				& list_box ! (
					@ connect_row_activated : move | _ , row | {
						on_film_sort_activated ( row , previous_sorting , & on_sort , sort_icons ) ;
					} ;
					& g_box ! (
						@ orientation : Horizontal ; @ spacing : 20 ;
						& label ! ( @ hexpand : true ; @ halign : Start ; "Name" ) ,
						& sort_icons [0] ,
					) ,
					& g_box ! (
						@ orientation : Horizontal ; @ spacing : 20 ;
						& label ! ( @ hexpand : true ; @ halign : Start ; "Release date" ) ,
						& sort_icons [1] ,
					) ,
					& g_box ! (
						@ orientation : Horizontal ; @ spacing : 20 ;
						& label ! ( @ hexpand : true ; @ halign : Start ; "Runtime" ) ,
						& sort_icons [2] ,
					) ,
				) ,
			) ;
			& label ! ("Sort") ,
		) ;

		Self { widget , previous_sorting }
	}
}
impl SeriesSortButton {
	pub fn new < F : Fn (SeriesSorting) + 'static > ( on_sort : F ) -> Self {
		let previous_sorting = leak ( RefCell :: new ( SeriesSorting :: default () ) ) ;

		let sort_icons = leak ( [
			image ! ( @ icon_name : "view-sort-ascending-symbolic" ; ) ,
			image ! ( @ icon_name : "view-sort-ascending-symbolic" ; ) ,
		] ) ;

		let widget = split_button ! (
			@ popover : & popover ! (
				@ css_classes : & [ "menu" ] ;
				& list_box ! (
					@ connect_row_activated : move | _ , row |
						on_series_sort_activated ( row , previous_sorting , & on_sort , sort_icons ) ;
					& g_box ! (
						@ orientation : Horizontal ; @ spacing : 20 ;
						& label ! ( @ hexpand : true ; @ halign : Start ; "Name" ) ,
						& sort_icons [0] ,
					) ,
					& g_box ! (
						@ orientation : Horizontal ; @ spacing : 20 ;
						& label ! ( @ hexpand : true ; @ halign : Start ; "First release date" ) ,
						& sort_icons [1] ,
					) ,
				) ,
			) ;
			& label ! ("Sort") ,
		) ;

		Self { widget , previous_sorting }
	}
}

impl Component <SplitButton> for FilmSortButton {
	fn get_widget ( & self ) -> & SplitButton { & self . widget }
}
impl Component <SplitButton> for SeriesSortButton {
	fn get_widget ( & self ) -> & SplitButton { & self . widget }
}

fn on_film_sort_activated < F : Fn (FilmsSorting) > (
	row : & ListBoxRow ,
	previous_sorting : & RefCell <FilmsSorting> ,
	on_sort : & F ,
	sort_icons : & [ Image ] ,
) {
	let sorting_property = match row . index () {
		0 => FilmProperty :: Name ,
		1 => FilmProperty :: ReleaseDate ,
		2 => FilmProperty :: Runtime ,
		_ => panic ! () ,
	} ;

	if sorting_property == previous_sorting . borrow () . property {
		let previous_sorting_direction = previous_sorting . borrow () . direction ;
		match previous_sorting_direction {
			Ascending => {
				previous_sorting . replace ( FilmsSorting :: new ( sorting_property , Descending ) ) ;
				for icon in sort_icons {
					icon . set_icon_name ( Some ("view-sort-descending-symbolic") ) ;
				}
				on_sort ( FilmsSorting :: new ( sorting_property , Descending ) ) ;
			} ,
			Descending => {
				previous_sorting . replace ( FilmsSorting :: new ( sorting_property , Ascending ) ) ;
				for icon in sort_icons {
					icon . set_icon_name ( Some ("view-sort-ascending-symbolic") ) ;
				}
				on_sort ( FilmsSorting :: new ( sorting_property , Ascending ) ) ;
			} ,
		}
	} else {
		previous_sorting . replace ( FilmsSorting :: new ( sorting_property , Ascending ) ) ;
		for icon in sort_icons {
			icon . set_icon_name ( Some ("view-sort-ascending-symbolic") ) ;
		}
		on_sort ( FilmsSorting :: new ( sorting_property , Ascending ) ) ;
	}
}
fn on_series_sort_activated < F : Fn (SeriesSorting) > (
	row : & ListBoxRow ,
	previous_sorting : & RefCell <SeriesSorting> ,
	on_sort : & F ,
	sort_icons : & [ Image ] ,
) {
	let sorting_property = match row . index () {
		0 => SeriesProperty :: Name ,
		1 => SeriesProperty :: FirstReleaseDate ,
		_ => panic ! () ,
	} ;

	if sorting_property == previous_sorting . borrow () . property {
		let previous_sorting_direction = previous_sorting . borrow () . direction ;
		match previous_sorting_direction {
			Ascending => {
				previous_sorting . replace ( SeriesSorting :: new ( sorting_property , Descending ) ) ;
				for icon in sort_icons {
					icon . set_icon_name ( Some ("view-sort-descending-symbolic") ) ;
				}
				on_sort ( SeriesSorting :: new ( sorting_property , Descending ) ) ;
			} ,
			Descending => {
				previous_sorting . replace ( SeriesSorting :: new ( sorting_property , Ascending ) ) ;
				for icon in sort_icons {
					icon . set_icon_name ( Some ("view-sort-ascending-symbolic") ) ;
				}
				on_sort ( SeriesSorting :: new ( sorting_property , Ascending ) ) ;
			} ,
		}
	} else {
		previous_sorting . replace ( SeriesSorting :: new ( sorting_property , Ascending ) ) ;
		for icon in sort_icons {
			icon . set_icon_name ( Some ("view-sort-ascending-symbolic") ) ;
		}
		on_sort ( SeriesSorting :: new ( sorting_property , Ascending ) ) ;
	}
}
