mod collated_grid ;
mod collation_menu ;

use gtk4 :: * ;
use gtk4 :: Orientation :: * ;
use gtk4 :: prelude :: * ;

use crate :: data_manager :: * ;
use crate :: ui :: component :: * ;
use crate :: ui :: utility :: * ;
use crate :: ui :: collatable_container :: collated_grid :: * ;
use crate :: ui :: collatable_container :: collation_menu :: * ;
use crate :: utility :: * ;



# [ derive ( Clone , Copy , PartialEq ) ] pub enum FilmProperty { Name , ReleaseDate , Runtime }
# [ derive ( Clone , Copy , PartialEq ) ] pub enum SeriesProperty { Name , FirstReleaseDate }
# [ derive ( Clone , Copy , PartialEq ) ] pub enum SortingDirection { Ascending , Descending }

# [ derive ( Clone , Copy ) ] pub struct FilmsSorting {
	property : FilmProperty ,
	direction : SortingDirection ,
}
# [ derive ( Clone , Copy ) ] pub struct SeriesSorting {
	property : SeriesProperty ,
	direction : SortingDirection ,
}

impl FilmsSorting {
	pub fn new ( property : FilmProperty , direction : SortingDirection ) -> Self {
		Self { property , direction }
	}
}
impl SeriesSorting {
	pub fn new ( property : SeriesProperty , direction : SortingDirection ) -> Self {
		Self { property , direction }
	}
}

impl Default for FilmsSorting {
	fn default () -> Self { Self {
		property : FilmProperty :: Name ,
		direction : SortingDirection :: Ascending ,
	} }
}
impl Default for SeriesSorting {
	fn default () -> Self { Self {
		property : SeriesProperty :: Name ,
		direction : SortingDirection :: Ascending ,
	} }
}



pub struct CollatableFilmsContainer {
	collated_grid : & 'static CollatedFilmsGrid ,
	widget : Box ,
}
pub struct CollatableSeriesContainer {
	collated_grid : & 'static CollatedSeriesGrid ,
	widget : Box ,
}

impl CollatableFilmsContainer {
	pub fn new () -> Self {
		let collated_grid = leak ( CollatedFilmsGrid :: new () ) ;
		let film_collation_menu = FilmCollationMenu :: new ( |sorting|
			collated_grid . set_sorting (sorting) ) ;

		let widget = g_box ! (
			@ orientation : Vertical ;
			film_collation_menu . get_widget () ,
			& scrolled_window ! (
				@ propagate_natural_height : true ;
				& vertically_filling ! ( collated_grid . get_widget () ) ,
			) ,
		) ;

		Self { collated_grid , widget }
	}

	pub async fn set_films ( & self , films : Vec <FilmOverview> ) {
		self . collated_grid . set_films ( films , FilmsSorting :: default () ) . await ;
	}
}
impl CollatableSeriesContainer {
	pub fn new () -> Self {
		let collated_grid = leak ( CollatedSeriesGrid :: new () ) ;
		let series_collation_menu = SeriesCollationMenu :: new ( |sorting|
			collated_grid . set_sorting (sorting) ) ;

		let widget = g_box ! (
			@ orientation : Vertical ;
			series_collation_menu . get_widget () ,
			& scrolled_window ! (
				@ propagate_natural_height : true ;
				& vertically_filling ! ( collated_grid . get_widget () ) ,
			) ,
		) ;

		Self { collated_grid , widget }
	}

	pub async fn set_series ( & self , series : Vec <SeriesOverview> ) {
		self . collated_grid . set_series ( series , SeriesSorting :: default () ) . await ;
	}
}

impl Component <Box> for CollatableFilmsContainer {
	fn get_widget ( & self ) -> & Box { & self . widget }
}
impl Component <Box> for CollatableSeriesContainer {
	fn get_widget ( & self ) -> & Box { & self . widget }
}
